/*
 * empathy-bus-names.h
 *
 * Copyright (C) 2014 Collabora Ltd. <http://www.collabora.co.uk/>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __EMPATHY_BUS_NAMES_H__
#define __EMPATHY_BUS_NAMES_H__

#define EMPATHY_BUS_NAME "org.gnome.Empathy"
#define EMPATHY_CALL_BUS_NAME "org.gnome.Empathy.Call"
#define EMPATHY_CHAT_BUS_NAME "org.gnome.Empathy.Chat"
#define EMPATHY_DEBUGGER_BUS_NAME "org.gnome.Empathy.Debugger"
#define EMPATHY_ACCOUNTS_BUS_NAME "org.gnome.EmpathyAccounts"

#endif /* #ifndef __EMPATHY_BUS_NAMES_H__*/
