.empathy-tab-close-button {
  -GtkButton-default-border : 0;
  -GtkButton-default-outside-border : 0;
  -GtkButton-inner-border: 0;
  -GtkWidget-focus-line-width : 0;
  -GtkWidget-focus-padding : 0;
  padding: 0;
}

.empathy-roster-window-error-button
{
  -GtkButtonBox-child-min-width: 0;
  -GtkButtonBox-child-min-height: 0;
  -GtkButtonBox-child-internal-pad-x: 0;
  -GtkButtonBox-child-internal-pad-y: 0;
}

.empathy-smiley-menu-item
{
  padding: 8px 12px;
}
